@@ -1,17 +1,22 @@
 """This module provides methods for handling queries."""
 import re
-from typing import List, Dict, Set, Tuple, Union, Any, Optional
+from typing import Callable, List, Dict, Set, Tuple, TypeVar, Union, Any, Optional
 from urllib.parse import quote
 from datetime import datetime
+import json
 
 from uvicorn.config import logger
 from botocore.exceptions import ClientError
+from ga4gh.vrsatile.pydantic.vrsatile_models import ValueObjectDescriptor
 
 from therapy import SOURCES, PREFIX_LOOKUP, ITEM_TYPES, NAMESPACE_LUIS
 from therapy.database import Database
-from therapy.schemas import Drug, SourceMeta, MatchType, ServiceMeta, \
-    HasIndication, SourcePriority, SearchService, NormalizationService, \
-    NamespacePrefix, SourceName
+from therapy.schemas import BaseNormalizationService, Drug, SourceMeta, MatchType, \
+    ServiceMeta, HasIndication, SourcePriority, SearchService, NormalizationService, \
+    NamespacePrefix, SourceName, UnmergedNormalizationService, MatchesNormalized
+
+
+NormService = TypeVar("NormService", bound=BaseNormalizationService)
 
 
 class InvalidParameterException(Exception):
@@ -80,6 +85,20 @@ class QueryHandler:
             self.db.cached_sources[src_name] = response
             return response
 
+    @staticmethod
+    def _get_indication(indication_string: str) -> HasIndication:
+        """Load indication data.
+        :param str indication_string: dumped JSON string from db
+        :return: complete HasIndication object
+        """
+        indication_values = json.loads(indication_string)
+        return HasIndication(
+            disease_id=indication_values[0],
+            disease_label=indication_values[1],
+            normalized_disease_id=indication_values[2],
+            supplemental_info=indication_values[3]
+        )
+
     def _add_record(self,
                     response: Dict[str, Dict],
                     item: Dict,
@@ -94,10 +113,7 @@ class QueryHandler:
         """
         inds = item.get("has_indication")
         if inds:
-            item["has_indication"] = [HasIndication(disease_id=i[0],
-                                                    disease_label=i[1],
-                                                    normalized_disease_id=i[2])
-                                      for i in inds]
+            item["has_indication"] = [self._get_indication(i) for i in inds]
 
         drug = Drug(**item)
         src_name = item["src_name"]
@@ -112,7 +128,9 @@ class QueryHandler:
                 "source_meta_": self._fetch_meta(src_name)
             }
         elif matches[src_name]["match_type"] == MatchType[match_type.upper()]:
-            matches[src_name]["records"].append(drug)
+            if drug.concept_id not in [r.concept_id for r
+                                       in matches[src_name]["records"]]:
+                matches[src_name]["records"].append(drug)
 
         return response, src_name
 
@@ -262,14 +280,14 @@ class QueryHandler:
         query = query.strip()
 
         # check if concept ID match
-        (response, sources) = self._check_concept_id(query, response, sources, infer)
+        response, sources = self._check_concept_id(query, response, sources, infer)
         if len(sources) == 0:
             return response
 
         query = query.lower()
         for match_type in ITEM_TYPES.values():
-            (response, sources) = self._check_match_type(query, response, sources,
-                                                         match_type)
+            response, sources = self._check_match_type(query, response, sources,
+                                                       match_type)
             if len(sources) == 0:
                 return response
 
@@ -298,8 +316,8 @@ class QueryHandler:
 
         return response_dict
 
-    def search_sources(self, query_str: str, keyed: bool = False, incl: str = "",
-                       excl: str = "", infer: bool = True) -> Dict:
+    def search(self, query_str: str, keyed: bool = False, incl: str = "",
+               excl: str = "", infer: bool = True) -> SearchService:
         """Fetch normalized therapy objects.
 
         :param str query_str: query, a string, to search for
@@ -359,23 +377,25 @@ class QueryHandler:
         response["service_meta_"] = ServiceMeta(
             response_datetime=datetime.now(),
         ).dict()
-        return SearchService(**response).dict()
+        return SearchService(**response)
 
-    def _add_merged_meta(self, response: Dict) -> Dict:
+    def _add_merged_meta(self, response: NormalizationService) -> NormalizationService:
         """Add source metadata to response object.
 
-        :param Dict response: in-progress response object
+        :param NormalizationService response: in-progress response object
         :return: completed response object.
         """
         sources_meta = {}
-        vod = response["therapy_descriptor"]
-        ids = [vod["therapy_id"]] + vod.get("xrefs", [])
+        vod = response.therapy_descriptor
+
+        xrefs = vod.xrefs or []  # type: ignore
+        ids = [vod.therapy_id] + xrefs  # type: ignore
         for concept_id in ids:
             prefix = concept_id.split(":")[0]
             src_name = PREFIX_LOOKUP[prefix.lower()]
             if src_name not in sources_meta:
                 sources_meta[src_name] = self._fetch_meta(src_name)
-        response["source_meta_"] = sources_meta
+        response.source_meta_ = sources_meta  # type: ignore
         return response
 
     def _record_order(self, record: Dict) -> Tuple[int, str]:
@@ -388,10 +408,10 @@ class QueryHandler:
         source_rank = SourcePriority[src]
         return source_rank, record["concept_id"]
 
-    def _add_vod(self, response: Dict, record: Dict, query: str,
-                 match_type: MatchType) -> Dict:
+    def _add_vod(self, response: NormalizationService, record: Dict, query: str,
+                 match_type: MatchType) -> NormalizationService:
         """Format received DB record as VOD and update response object.
-        :param Dict response: in-progress response object
+        :param NormalizationService response: in-progress response object
         :param Dict record: record as stored in DB
         :param str query: query string from user request
         :param MatchType match_type: type of match achieved
@@ -410,8 +430,7 @@ class QueryHandler:
         if "aliases" in record:
             vod["alternate_labels"] = record["aliases"]
 
-        if any(filter(lambda f: f in record, ("approval_rating",
-                                              "approval_ratings",
+        if any(filter(lambda f: f in record, ("approval_ratings",
                                               "approval_year",
                                               "has_indication"))):
             approv = {
@@ -419,13 +438,7 @@ class QueryHandler:
                 "name": "regulatory_approval",
                 "value": {}
             }
-            # temporarily handle rating vs ratings fields slightly differently
-            # to change in issue 223
-            if "approval_rating" in record:
-                value = record.get("approval_rating")
-                if value:
-                    approv["value"]["approval_ratings"] = [value]  # type: ignore
-            elif "approval_ratings" in record:
+            if "approval_ratings" in record:
                 value = record.get("approval_ratings")
                 if value:
                     approv["value"]["approval_ratings"] = value  # type: ignore
@@ -433,16 +446,27 @@ class QueryHandler:
                 value = record.get("approval_year")
                 if value:
                     approv["value"]["approval_year"] = value  # type: ignore
+
             inds = record.get("has_indication", [])
             inds_list = []
-            for ind in inds:
-                ind_obj = {
-                    "id": ind[0],
+            for ind_db in inds:
+                indication = self._get_indication(ind_db)
+                ind_value_obj: Dict[str, Optional[Union[str, List]]] = {
+                    "id": indication.disease_id,
                     "type": "DiseaseDescriptor",
-                    "label": ind[1],
-                    "disease_id": ind[2],
+                    "label": indication.disease_label,
+                    "disease_id": indication.normalized_disease_id,
                 }
-                inds_list.append(ind_obj)
+                if indication.supplemental_info:
+                    ind_value_obj["extensions"] = [
+                        {
+                            "type": "Extension",
+                            "name": k,
+                            "value": v
+                        }
+                        for k, v in indication.supplemental_info.items()
+                    ]
+                inds_list.append(ind_value_obj)
             if inds_list:
                 approv["value"]["has_indication"] = inds_list  # type: ignore
             vod["extensions"].append(approv)
@@ -461,104 +485,188 @@ class QueryHandler:
         if not vod["extensions"]:
             del vod["extensions"]
 
-        response["match_type"] = match_type
-        response["therapy_descriptor"] = vod
+        response.match_type = match_type
+        response.therapy_descriptor = ValueObjectDescriptor(**vod)  # type: ignore
         response = self._add_merged_meta(response)
         return response
 
-    def _handle_failed_merge_ref(self, record: Dict, response: Dict,
-                                 query: str) -> Dict:
-        """Log + fill out response for a failed merge reference lookup.
-
-        :param Dict record: record containing failed merge_ref
-        :param Dict response: in-progress response object
-        :param str query: original query value
-        :return: response with no match
-        """
-        logger.error(f"Merge ref lookup failed for ref {record['merge_ref']} "
-                     f"in record {record['concept_id']} from query `{query}`")
-        response["match_type"] = MatchType.NO_MATCH
-        return response
-
-    def _resolve_merge(self, response: Dict, query: str,
-                       record: Dict, match_type: MatchType) -> Dict:
+    def _resolve_merge(
+        self, response: NormService, query: str, record: Dict,
+        match_type: MatchType, callback: Callable
+    ) -> NormService:
         """Given a record, return the corresponding normalized record
 
-        :param Dict response: in-progress response object
+        :param NormalizationService response: in-progress response object
         :param str query: exact query as provided by user
         :param Dict record: record to retrieve normalized concept for
         :param MatchType match_type: type of match that returned these records
-        :return: formed response, a Dict
+        :param Callable callback: response constructor method
+        :return: Normalized response object
         """
         merge_ref = record.get("merge_ref")
         if merge_ref:
             # follow merge_ref
             merge = self.db.get_record_by_id(merge_ref, False, True)
             if merge is None:
-                return self._handle_failed_merge_ref(record, response, query)
+                logger.error(f"Merge ref lookup failed for ref {record['merge_ref']} "
+                             f"in record {record['concept_id']} from query `{query}`")
+                return response
             else:
-                return self._add_vod(response, merge, query, match_type)
+                return callback(response, merge, match_type)
         else:
             # record is sole member of concept group
-            return self._add_vod(response, record, query, match_type)
+            return callback(response, record, match_type)
 
-    def search_groups(self, query: str, infer: bool = True) -> Dict:
+    def _prepare_normalized_response(self, query: str) -> Dict[str, Any]:
+        """Provide base response object for normalize endpoints.
+
+        :param str query: user-provided query
+        :return: basic normalization response boilerplate
+        """
+        return {
+            "query": query,
+            "match_type": MatchType.NO_MATCH,
+            "warnings": self._emit_char_warnings(query),
+            "service_meta_": ServiceMeta(response_datetime=datetime.now())
+        }
+
+    def _get_matches_by_type(self, query: str, match_type: str) -> List[Dict]:
+        """Get matches list for match tier.
+        :param str query: user-provided query
+        :param str match_type: keyword of match type to check
+        :return: List of records matching the query and match level
+        """
+        matching_refs = self.db.get_records_by_type(query, match_type)
+        matching_records = [self.db.get_record_by_id(m["concept_id"], False)
+                            for m in matching_refs]
+        return sorted(matching_records, key=self._record_order)  # type: ignore
+
+    def normalize(self, query: str, infer: bool = True) -> NormalizationService:
         """Return merged, normalized concept for given search term.
 
         :param str query: string to search against
         :param bool infer: if true, try to infer namespace for IDs
+        :return: Normalized response object
         """
         # prepare basic response
-        response: Dict[str, Any] = {
-            "query": query,
-            "warnings": self._emit_char_warnings(query),
-            "service_meta_": ServiceMeta(
-                response_datetime=datetime.now(),
-            ).dict()
-        }
+        response = NormalizationService(**self._prepare_normalized_response(query))
+
+        add_vod_curry = lambda res, rec, mat: self._add_vod(res, rec, query, mat)  # noqa: E501 E731
+        return self._perform_normalized_lookup(response, query, infer, add_vod_curry)
+
+    def _construct_drug_match(self, record: Dict) -> Drug:
+        """Create individual Drug match for unmerged normalization endpoint.
+
+        :param Dict record: record to add
+        :return: completed Drug object
+        """
+        inds = record.get("has_indication")
+        if inds:
+            record["has_indication"] = [self._get_indication(i) for i in inds]
+        return Drug(**record)
+
+    def _add_normalized_records(self, response: UnmergedNormalizationService,
+                                normalized_record: Dict,
+                                match_type: MatchType) -> UnmergedNormalizationService:
+        """Add individual records to unmerged normalize response.
+
+        :param UnmergedNormalizationService response: in-progress response object
+        :param Dict normalized_record: record associated with normalized concept,
+            either merged or single identity
+        :param MatchType match_type: type of match achieved
+        :return: Completed response object
+        """
+        response.match_type = match_type
+        response.normalized_concept_id = normalized_record["concept_id"]
+        if normalized_record["item_type"] == "identity":
+            record_source = SourceName[normalized_record["src_name"].upper()]
+            response.source_matches[record_source] = MatchesNormalized(
+                records=[self._construct_drug_match(normalized_record)],
+                source_meta_=self._fetch_meta(record_source.value)
+            )
+        else:
+            concept_ids = [normalized_record["concept_id"]] + \
+                normalized_record.get("xrefs", [])
+            for concept_id in concept_ids:
+                record = self.db.get_record_by_id(concept_id, case_sensitive=False)
+                if not record:
+                    continue  # cover a few chemidplus edge cases
+                record_source = SourceName[record["src_name"].upper()]
+                drug = self._construct_drug_match(record)
+                if record_source in response.source_matches:
+                    response.source_matches[record_source].records.append(drug)
+                else:
+                    response.source_matches[record_source] = MatchesNormalized(
+                        records=[drug],
+                        source_meta_=self._fetch_meta(record_source.value)
+                    )
+        return response
+
+    def _perform_normalized_lookup(self, response: NormService, query: str, infer: bool,
+                                   response_builder: Callable) -> NormService:
+        """Retrieve normalized concept, for use in normalization endpoints
+        :param NormService response: in-progress response object
+        :param str query: user-provided query
+        :param bool infer: whether to try namespace inference
+        :param Callable response_builder: response constructor callback method
+        :return: completed service response object
+        """
         if query == "":
-            response["match_type"] = MatchType.NO_MATCH.value
             return response
         query_str = query.lower().strip()
 
         # check merged concept ID match
         record = self.db.get_record_by_id(query_str, case_sensitive=False, merge=True)
         if record:
-            return self._add_vod(response, record, query, MatchType.CONCEPT_ID)
+            return response_builder(response, record, MatchType.CONCEPT_ID)
 
         # check concept ID match
         record = self.db.get_record_by_id(query_str, case_sensitive=False)
         if record:
-            return self._resolve_merge(response, query, record,
-                                       MatchType.CONCEPT_ID)
+            return self._resolve_merge(response, query, record, MatchType.CONCEPT_ID,
+                                       response_builder)
 
         # check concept ID match with inferred namespace
         if infer:
             inferred_response = self._infer_namespace(query)
             if inferred_response:
-                response = self._resolve_merge(response, query,
-                                               inferred_response[0],
-                                               MatchType.CONCEPT_ID)
-                response["warnings"].append(inferred_response[1])
-                return response
+                if response.warnings:
+                    response.warnings.append(inferred_response[1])
+                else:
+                    response.warnings = [inferred_response[1]]
+                return self._resolve_merge(response, query, inferred_response[0],
+                                           MatchType.CONCEPT_ID,
+                                           response_builder)
 
         # check other match types
         for match_type in ITEM_TYPES.values():
-            # get matches list for match tier
-            matching_refs = self.db.get_records_by_type(query_str, match_type)
-            matching_records = \
-                [self.db.get_record_by_id(m["concept_id"], False)
-                 for m in matching_refs]
-            matching_records.sort(key=self._record_order)  # type: ignore
+            matching_records = self._get_matches_by_type(query_str, match_type)
 
             # attempt merge ref resolution until successful
             for match in matching_records:
                 assert match is not None
                 record = self.db.get_record_by_id(match["concept_id"], False)
                 if record:
+                    match_type_value = MatchType[match_type.upper()]
                     return self._resolve_merge(response, query, record,
-                                               MatchType[match_type.upper()])
+                                               match_type_value,
+                                               response_builder)
 
-        if not matching_records:  # type: ignore
-            response["match_type"] = MatchType.NO_MATCH.value
-        return NormalizationService(**response).dict()
+        return response
+
+    def normalize_unmerged(
+        self, query: str, infer: bool = True
+    ) -> UnmergedNormalizationService:
+        """Return all source records under the normalized concept for the provided
+        query string.
+
+        :param str query: string to search against
+        :param bool infer: if true, try to infer namespace for IDs
+        :return: Normalized response object
+        """
+        response = UnmergedNormalizationService(
+            source_matches={},
+            **self._prepare_normalized_response(query)
+        )
+        return self._perform_normalized_lookup(response, query, infer,
+                                               self._add_normalized_records)
